(function(window, document, undefined) {
    var
            HEADEND_PING_INTERVAL = 3000, // ms

            // Tokens
            ACTIVE = 'active',
            INACTIVE = 'inactive',
            DISABLED = 'disabled',
            ONLINE = 'online',
            OFFLINE = 'offline',
            NORMAL = 'normal',
            EMERGENCY = 'emergency',
            // Cached DOM objects 
            $alertState = $('.AlertState'),
            $stateLabel = $('.StateLabel'),
            App = function() {

                // Headend connection state
                this.isConnected = null;

                // Headend state
                this.headendStae = null;

                this.$headendTitle = $('.HeadendTitle');

                // Password checker box
                this.passwordChecker = null;

                // Alert message box
                this.popupMessage = null;

                this.start = App.prototype.start.bind(this);

            };





    /**
     * Application launching point.
     * 
     * @returns {undefined}
     */
    App.prototype.start = function() {
        var
                _this = this,
                fxapp = window.jfx;

        // Initialize view components
        _this.setHeadendTitle();
        _this.showHeadendSwitcher(INACTIVE);

        // Setup headend online/offline state indicator
        _this.checkOnlineState();
        setInterval(_this.checkOnlineState.bind(_this), HEADEND_PING_INTERVAL);

        // Switch station to emergency mode handler
        $('.StateLabel.normal').on('click', function(ev) {
            if (_this.isConnected === true) {
                _this._switchHeadend.call(_this, EMERGENCY);
            }
        }.bind(_this));

        // Switch station to normal mode handler
        $('.StateLabel.emergency, .StateLabel.inactive').on('click', function(ev) {
            if (_this.isConnected === true) {
                _this._switchHeadend.call(_this, NORMAL);
            }
        }.bind(_this));

        // Window control - Close button
        $('.WinButtons .Button.Close').on('click',
                function() {
                    window.jfx.closeApp();
                });

        // Window control - Minimize button
        $('.WinButtons .Button.Minimize').on('click',
                function() {
                    window.jfx.minimizeApp();
                });


        _this.passwordChecker = new PasswordChecker('.Password');

        _this.popupMessage = new PopupMessage('.PopupMessage');

        _this.disableContextMenu();

        // Emulate clicking on switcher to set headend in normal state on startup
        $('.StateLabel.inactive').trigger('click');
    }



    /**
     * Checks headend online/offline state and set application flag
     * @returns {undefined}
     */
    App.prototype.checkOnlineState = function() {
        var app = window.jfx;

        if (app.headend.isConnected() === true) {
            this.isConnected = true;
            this._showState(ONLINE);
            $stateLabel.removeClass(DISABLED);
        }
        else {
            this.isConnected = false;
            this._showState(OFFLINE);
            $stateLabel.addClass(DISABLED);
        }
    }




    /**
     * Set Headend title
     * @param {type} title
     * @returns {App} Self
     */
    App.prototype.setHeadendTitle = function() {
        var
                title = this._getHeadendTitle();

        if (title === undefined) {
            title = '';
        }
        this.$headendTitle.html(title);

        return this;
    }







    /**
     * Set online indicator to given state
     * @param {string} state {'online' | 'offline'}     
     * @return {App} Self
     */
    App.prototype._showState = function(state) {
        $('.OnlineIndicator').find('article').removeClass(ACTIVE);


        if (state !== undefined) {
            if (state !== ONLINE && state !== OFFLINE) {
                state = OFFLINE;
            }
            $('.OnlineIndicator').find('.' + state).addClass(ACTIVE);
        }

        return this;
    }




    App.prototype.showHeadendSwitcher = function(switchType) {
        if (switchType === undefined || switchType !== NORMAL && switchType !== EMERGENCY) {
            switchType = INACTIVE;
        }
        $('.StateLabel').removeClass(ACTIVE);
        $('.StateLabel.' + switchType).addClass(ACTIVE);
        return this;

    }



    /**
     * Reads headend title from Java side
     * 
     * @returns {String} Headend title
     */
    App.prototype._getHeadendTitle = function() {
        var title = window.jfx.headend.getHeadendName();
        return title;
    }





    /**
     * Switch headend to given mode using Java API invocation
     * @param {String} mode ['normal', 'emergency']
     * @returns {App}
     */
    App.prototype._switchHeadend = function(mode) {
        var
                _this = this,
                fxapp = window.jfx;

        // Ask for password
        this.passwordChecker.checkPassword(
                function() {
                    switch (mode) {
                        case NORMAL:
                            // Normal broadcast mode
                            _this.showHeadendSwitcher(NORMAL);
                            fxapp.headend.switchToNormal();
                            $alertState.hide();  // Normal broadcast, hide alert state
                            break;

                        case EMERGENCY:
                            // Emergency broadcast mode
                            _this.showHeadendSwitcher(EMERGENCY);
                            fxapp.headend.switchToEmergency();
                            $alertState.show();  // Broadcast translation, show alert state
                            break;
                        default:
                    }
                },
                function() {
                    _this.popupMessage.setText('Неверный пароль/пароль не введён').show(1500);
                }
        );
    };




    App.prototype.disableContextMenu = function() {
        $(window).on('contextmenu', function(ev) {
            ev.preventDefault();
            return false;
        });
    };

    window.app = new App();
})(window, document);




$(document).ready(function() {
    alert('JavaScript part started...');
    app.start.call(app);
}); 