/**
 * Password check message window
 */

(function() {


    var
            // tokens
            ACTIVE = 'active',
            /**
             * Password check class
             * 
             * @constructor
             * @param {string} formClass
             * @returns {PasswordChecker}
             */
            PasswordChecker = function(formClass) {
                /**
                 * Password input form (input + button)
                 */
                this.$form = $(formClass);

                /**
                 * Callback for correct password
                 * @returns {undefined}
                 */
                this.onOk = function() {
                };

                /**
                 * Callback for incorrect password
                 * @returns {undefined}
                 */
                this.onFail = function() {
                };

                /**
                 * Buffer for entered password
                 */
                this.passwordBuffer = null;

                // Initialize
                this.initHandlers();
            }




    /**
     * Initialize all handlers for user interaction
     * 
     * @returns {undefined}
     */
    PasswordChecker.prototype.initHandlers = function() {
        var
                _this = this;

        // Capture password field while typing
        this.$form.find('input').on('keyup',
                function() {
                    _this.passwordBuffer = $(this).val();
                });

        // Escape button cancels checking
        this.$form.on('keyup', function(ev) {
            // Hide form on escape and set result as false
            if (ev.keyCode === 27) {
                _this.hide();
                _this.onFail();
            }

        });

        // Send password button
        this.$form.find('button').on('click keydown',
                function() {
                    var checkResult = _this._checkPassword();
                    if (checkResult === true) {
                        _this.onOk();
                    }
                    else {
                        _this.onFail();
                    }
                    _this.hide();
                });

    };




    /**
     * Show password dialogue box
     * @returns {undefined}
     */
    PasswordChecker.prototype.show = function() {
        if (this.$form !== null) {
            this.clearInput(); // clear input field
            this.$form.addClass(ACTIVE);
        }
    };




    /**
     * Hide password dialogue box
     * @returns {undefined}
     */
    PasswordChecker.prototype.hide = function() {
        if (this.$form !== null) {
            this.$form.removeClass(ACTIVE);
        }
    };




    /**
     * Clear input field for password
     * @returns {undefined}
     */
    PasswordChecker.prototype.clearInput = function() {
        this.$form.find('input').val('');
        this.passwordBuffer = null;
    };


    /**
     * Check password and execute `ok` in case of password correct and `fail`
     * in case of password fail.
     * 
     * @param {Function} ok Callback for correct password
     * @param {Function} fail Callback for incorrect password
     * @returns {undefined}
     */
    PasswordChecker.prototype.checkPassword = function(ok, fail) {
        var
                _this = this;

        _this.onOk = ok;
        _this.onFail = fail;

        $(_this.$form.find('input').get(0)).focus();
        _this.show();
    };






    /**
     * Check currently input password with application password.
     * 
     * @returns {boolean} True if password is OK, False if password incorrect
     */
    PasswordChecker.prototype._checkPassword = function() {
        var
                result = true,
                password = this.passwordBuffer;

        if (password === null) {
            password = '';
        }
        alert('password ' + password + ' sent to backend for checking');
        result = window.jfx.headend.isPasswordCorrect(password);

        return result;
    }






    window.PasswordChecker = PasswordChecker;
})(window);

